#!/usr/bin/perl
package SimpleObjectRegistry;
use strict;
use warnings;
use SimpleObject;
use Storable qw(dclone store retrieve);
use Data::Dumper;
use Carp qw(cluck croak);

our @ISA = qw(SimpleObject);

sub counter {
	$_[0]->{count};
}

 sub create {
	my $class = shift;
	my $self = {};
	$self->{list} = [];
	$self->{lookup} = {};
	$self->{count} = 0;
	bless $self, $class;
 }
 
 sub add_objects {
	my $self = shift;
	my @array = @_;
	eval {
		local $SIG{__WARN__}=sub{};
		while (@array) {
			my $co	= shift @array;	
			croak "$co is not a reference\n" unless (ref $co); 	
			my $id = $co->get_attribute("id");
			push @{$self->{list}}, $co;
			$self->{lookup}->{$id} = $#{$self->{list}};
			$self->{count} += 1;
			
		}
		
	};
	croak $@ if ($@);
 }
 sub add_or_replace_objects {
	my $self = shift;
	my @array = @_;
	eval {
		local $SIG{__WARN__}=sub{};
		while (@array) {
			my $co	= shift @array;	
			croak "$co is not a reference\n" unless (ref $co); 	
			my $id = $co->get_attribute("id");
			if (exists $self->{lookup}->{$id} ) {
				my $index = $self->{lookup}->{$id};
				$self->{list}->[$index] = $co;
			}	
			else {
				push @{$self->{list}}, $co;
				$self->{lookup}->{$id} = $#{$self->{list}};
				$self->{count} += 1;
			}
		}
		
	};
	croak $@ if ($@);
 }
 sub delete_objects {
	my $self = shift;
	while (@_) {
		my $co	= shift @_;	
		my $id = $co->get_attribute("id");
		next unless (exists $self->{lookup}->{$id});
		my $index = $self->{lookup}->{$id};
		my @lowerpart = @{$self->{list}}[0 .. $index -1];
		my @upperpart = @{$self->{list}}[$index +1 .. $self->{count}-1];
		$self->{list} = \@lowerpart;
		push @{$self->{list}}, @upperpart;
		delete $self->{lookup}->{$id};
		for my $i (@upperpart) {
		      my $idi = $i->get_attribute("id");
		      $self->{lookup}->{$idi} -= 1;
		}
		$self->{count} -= 1;
	}
 }
sub clear {
	my $self = shift;
	$self->{list} =[];
	$self->{lookup} = {};
	$self->{count} = 0;
	1;
}
sub shift_object {
	my $self = shift;
	my $shiftedobject = shift @{$self->{list}};
	my $id = $shiftedobject->get_attribute("id");
	delete $self->{lookup}->{$id};
	$self->{count} -= 1;
	foreach my $i (keys %{$self->{lookup}}) {
		$self->{lookup}->{$id} -= 1;
	}
	return $shiftedobject;
}

 sub get_object_by_id {
	my $index = $_[0]->{lookup}->{$_[1]};
	if (defined $index) {
		return $_[0]->{list}[$index] ;
	} else {
		return undef;
	}
 }
sub object_list {
	my $self = shift;
	return @{$self->{list}};
}
sub objects_hash_by_id {
	my ($self) = @_;
	my @result = map { ($_->id()  => $_ )} @{$self->{list}};
	return \@result
}
sub object_list_by_attribute {
	my $self = shift;
	my $attribute = shift;
	my $hashofarrays= {};
	
	for my $i ( @{$self->{list}} ) {
		my $group = $i->get_attribute($attribute);
		push @{$hashofarrays->{$group}}, $i;
	};
	return $hashofarrays;
}
sub object_list_if_attribute {
	my $self = shift;
	my $attribute = shift;
	my $condition = shift;
	my @array;
	
	for my $i ( @{$self->{list}} ) {
		my $x = $i->get_attribute($attribute);
		my $teststring = $x." ".$condition;
		push @array, $i if (eval($teststring));
	};
	return @array;
}

sub save {
	my $self = shift;
	my $file = shift;
	store $self, $file || warn "Could not store!";
	1;
}
sub recreate {
	my $class = shift;
	my $file = shift;
	my $self = retrieve ($file) ;
	croak ("Error: Could not retrieve data from $file!\n") unless (defined $self);
	bless $self, $class;
	return $self;
}
sub summarize_attribute {
	my $self = shift;
	my $attribute = shift;
	my $aggregate = shift;
	my $result;
	if ($aggregate eq "total") {
		$result = 0;
		for my $i ($self->object_list()) {
			
			$result += $i->get_attribute($attribute);
		}
	}
	elsif ($aggregate eq "groupcount") {
		for my $i ($self->object_list()) {
			my $group = $i->get_attribute($attribute);
			$result->{$group} = 0 unless (exists $result->{$group});
			$result->{$group} += 1;
		}
	}
	
	return $result;
}
sub save_attribute_table_as_ascii {
	my $self = shift;
	my $fname = shift;
	my @attributes = @_;
	print "Saving $fname...\n";
	open (my $fh, ">", $fname) or die "$!";
	print $fh join("\t", ("id", @attributes) )."\n" ;
	for my $i (sort {$a->get_attribute("id") cmp $b->get_attribute("id")} @{$self->{list}}) {
		print $fh $i->get_attribute("id");
		for my $j (@attributes) {
			print $fh "\t";
			print $fh $i->get_attribute($j);
		}
		print $fh "\n";
	}
	close ($fh);
	return 1;
}
1;
